@@ -16,14 +16,19 @@ import math
 import collections
 
 # Change matplotlib formatting. TODO incorporate into a style?
-plt.rcParams["patch.force_edgecolor"] = True
+plt.rcParams['patch.force_edgecolor'] = True
 
 def make_array(*elements):
     """Returns an array containing all the arguments passed to this function.
     A simple way to make an array with a few elements.
 
     As with any array, all arguments should have the same type.
-
+    
+    Args:
+           ``elements`` (variadic): elements 
+    Returns:
+           A NumPy array of same length as the provided varadic argument ``elements``
+           
     >>> make_array(0)
     array([0])
     >>> make_array(2, 3, 4)
@@ -31,12 +36,15 @@ def make_array(*elements):
     >>> make_array("foo", "bar")
     array(['foo', 'bar'],
           dtype='<U3')
+    >>> make_array(True, False)
+    array([ True, False], dtype=bool)
     >>> make_array()
     array([], dtype=float64)
     """
-    if elements and all(isinstance(item, (int, np.integer)) for item in elements):
+    if elements and all(isinstance(item, (int, np.integer)) and not isinstance(item, bool) for item in elements):
         # Specifically added for Windows machines where the default 
         # integer is int32 - see GH issue #339.
+        # and ensures item is not bool, see issue #622.
         return np.array(elements, dtype="int64")
 
     # Manually cast `elements` as an object due to this: https://github.com/data-8/datascience/issues/458
@@ -118,10 +126,10 @@ def plot_normal_cdf(rbound=None, lbound=None, mean=0, sd=1):
 plot_cdf_area = plot_normal_cdf
 
 
-def sample_proportions(sample_size, probabilities):
+def sample_proportions(sample_size: int, probabilities, seed=None):
     """Return the proportion of random draws for each outcome in a distribution.
 
-    This function is similar to np.random.multinomial, but returns proportions
+    This function is similar to np.random.Generator.multinomial, but returns proportions
     instead of counts.
 
     Args:
@@ -129,19 +137,22 @@ def sample_proportions(sample_size, probabilities):
 
         ``probabilities``: An array of probabilities that forms a distribution.
 
+        ``seed``: Optional seed for reproducibility. If None, results will be random.
+
     Returns:
         An array with the same length as ``probability`` that sums to 1.
     """
-    return np.random.multinomial(sample_size, probabilities) / sample_size
+    rng = np.random.default_rng(seed)
+    return rng.multinomial(sample_size, probabilities) / sample_size
 
 
 def proportions_from_distribution(table, label, sample_size,
-                                  column_name='Random Sample'):
+                                  column_name='Random Sample', seed=None):
     """
     Adds a column named ``column_name`` containing the proportions of a random
     draw using the distribution in ``label``.
 
-    This method uses ``np.random.multinomial`` to draw ``sample_size`` samples
+    This method uses ``np.random.Generator.multinomial`` to draw ``sample_size`` samples
     from the distribution in ``table.column(label)``, then divides by
     ``sample_size`` to create the resulting column of proportions.
 
@@ -156,6 +167,8 @@ def proportions_from_distribution(table, label, sample_size,
         ``column_name``: The name of the new column that contains the sampled
             proportions. Defaults to ``'Random Sample'``.
 
+        ``seed``: Optional seed for reproducibility. If None, results will be random.
+
     Returns:
         A copy of ``table`` with a column ``column_name`` containing the
         sampled proportions. The proportions will sum to 1.
@@ -164,8 +177,8 @@ def proportions_from_distribution(table, label, sample_size,
         ``ValueError``: If the ``label`` is not in the table, or if
             ``table.column(label)`` does not sum to 1.
     """
-    proportions = sample_proportions(sample_size, table.column(label))
-    return table.with_column('Random Sample', proportions)
+    proportions = sample_proportions(sample_size, table.column(label), seed)
+    return table.with_column(column_name, proportions)
 
 
 def table_apply(table, func, subset=None):
@@ -254,6 +267,5 @@ def is_non_string_iterable(value):
         return False
     if hasattr(value, '__iter__'):
         return True
-    if isinstance(value, collections.abc.Sequence):
-        return True
-    return False
\ No newline at end of file
+    return False
+
