@@ -6,6 +6,7 @@ import numpy as np
 from numpy.testing import assert_array_equal
 from datascience import *
 import pandas as pd
+from io import BytesIO
 
 
 #########
@@ -547,6 +548,11 @@ def test_groups_nans(table5):
     3          | 10     | 1
     """)
 
+def test_groups_with_nonexistent_label(table):
+    t = table.copy()
+    with (pytest.raises(ValueError)):
+        t.groups(['bad_label', 'points'])
+
 def test_join(table, table2):
     """Tests that join works, not destructive"""
     t = table
@@ -572,6 +578,12 @@ def test_join(table, table2):
     z      | 1     | 10     | 10
     """)
 
+def test_set_format_with_no_format_column(mocker, table):
+    MockNumberFormatter = mocker.NonCallableMock(spec=NumberFormatter)
+    del MockNumberFormatter.format_column
+    with pytest.raises(Exception):
+        table.set_format('count', MockNumberFormatter)
+
 def test_join_html(table, table2):
     """Test that join doesn't crash with formatting."""
     t = table
@@ -664,6 +676,9 @@ def test_apply(table):
                        np.array([18, 6, 6, 2]))
     with(pytest.raises(ValueError)):
         t.apply(lambda x, y: x + y, 'count', 'score')
+    with(pytest.warns(FutureWarning)):
+        assert_array_equal(t.apply(lambda x: x * x, ['points']),
+                           np.array([1, 4, 4, 100]))
 
     # Deprecated behavior
     assert_array_equal(t.apply(lambda x, y: x * y, 'count', 'points'),
@@ -768,6 +783,18 @@ def test_append_row(table):
     g      | 2     | 2
     """)
 
+def test_append_none(table):
+    row = [[], None, "", 0]
+    for i in row:
+        table.append(i)
+        assert_equal(table, """
+                     letter | count | points
+                     a      | 9     | 1
+                     b      | 3     | 2
+                     c      | 3     | 2
+                     z      | 1     | 10
+                     """)
+
 def test_append_row_by_array(table):
     row = np.array(['g', 2, 2])
     table.append(row)
@@ -1131,6 +1158,11 @@ def test_remove_single(table):
     z      | 1     | 10
     """)
 
+def test_remove_none(table):
+    num_rows_before = table.num_rows
+    assert table.remove([]) is None
+    assert table.num_rows == num_rows_before
+
 def test_remove_zeroth_row(table):
     table.remove(0)
     assert_equal(table, """
@@ -1145,6 +1177,13 @@ def test_remove_num_rows(table):
     table.remove(0)
     assert table.num_rows + 1 == num_rows_before
 
+def test_remove_column_type(table):
+    original_col = table.column("count")
+    assert type(original_col) == np.ndarray
+    table.remove(0)
+    assert type(table.column("count")) == type(original_col)
+
+
 
 ##########
 # Create #
@@ -1556,6 +1595,15 @@ def test_join_conflicting_column_names(table, table4):
     c      | 3     | 2      | 6      | 2       | 2               | Epsilon
     """)
 
+def test_join_no_rows():
+    t1 = Table(['letter'])
+    t2 = Table(['letter'])
+    t2.append(['b'])
+    t3 = t1.join('letter', t2)
+    assert t3 == None
+    t4 = t2.join('letter', t1)
+    assert t4 == None
+    
 def test_percentile(numbers_table):
     assert_equal(numbers_table.percentile(76), """
     count | points
@@ -1583,6 +1631,36 @@ def test_move_column(table):
     table = table.move_column(2, 1)
     assert table.labels == ('count', 'points', 'letter')
 
+
+def test_join_with_empty_self(table2):
+    empty_table = Table().with_columns(['letter', []])
+    result = empty_table.join('letter', table2)
+    assert result is None, "Expected None when joining with an empty self table"
+
+def test_join_with_empty_other(table):
+    empty_table = Table().with_columns(['points', []])
+    result = table.join('points', empty_table)
+    assert result is None, "Expected None when joining with an empty other table"
+
+def test_join_with_other_label(table, table2):
+    result = table.join('points', table2, 'points')
+    assert_equal(result, """
+    points | letter | count | names
+    1      | a      | 9     | one
+    2      | b      | 3     | two
+    2      | c      | 3     | two
+    """)
+
+def test_join_without_other_label(table, table2):
+    result = table.join('points', table2)
+    assert_equal(result, """
+    points | letter | count | names
+    1      | a      | 9     | one
+    2      | b      | 3     | two
+    2      | c      | 3     | two
+    """)
+    
+    
 ##################
 # Export/Display #
 ##################
@@ -1894,3 +1972,26 @@ def test_read_table():
     assert isinstance(Table().read_table("tests/us-unemployment-copy"), Table)
     assert isinstance(Table().read_table("tests/us-unemployment.txt"), Table)
     assert isinstance(Table().read_table("https://raw.githubusercontent.com/data-8/textbook/main/assets/data/deflategate.csv"), Table)
+
+    """Test that Tables reads csv content from a buffer."""
+    buffer = BytesIO()
+    with open('tests/us-unemployment.csv', 'rb') as f:
+        buffer.write(f.read())
+        buffer.seek(0)
+        assert isinstance(Table().read_table(buffer), Table)
+
+def test_no_records():
+    """Test that Table returns nothing on an empty record"""
+    empty_table = Table()
+    records_empty_table = Table().from_records([])
+    assert empty_table == records_empty_table
+
+def test_column_index(table):
+    """Test that Tables returns right index number"""
+    index = table.column_index('letter')
+    assert index == 0
+
+def test_num_columns(table):
+    """Test that Tables returns right number of columns"""
+    number = table.num_columns
+    assert number == 3
